'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _pluralize = require('pluralize');

var _pluralize2 = _interopRequireDefault(_pluralize);

var _utils = require('./utils.js');

var _controllers = require('./controllers.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var stateParamPatterns = {
    int: '\\d+',
    uuid: '[0-9a-fA-F]{8}(?:-[0-9a-fA-F]{4}){3}-[0-9a-fA-F]{12}',
    str: '\\S+'
};

function tryLoadTemplate(requireContext, template) {
    try {
        return requireContext(template);
    } catch (e) {}
}

function defineRoute(_ref) {
    var name = _ref.name;
    var parent = _ref.parent;
    var displayName = _ref.displayName;
    var stateParamKey = _ref.stateParamKey;
    var stateParamPattern = _ref.stateParamPattern;
    var listConfig = _ref.list;
    var createConfig = _ref.create;
    var viewConfig = _ref.view;
    var editConfig = _ref.edit;

    var baseName = parent ? parent + (0, _utils.capFirst)(name) : name;
    var list = baseName;
    var create = baseName + 'Create';
    var view = baseName + 'View';
    var edit = view + 'Edit';
    var routeDefinition = {
        list: listConfig ? list : null,
        create: createConfig ? create : null,
        view: viewConfig ? view : null,
        edit: editConfig ? edit : null,
        stateParamPattern: stateParamPatterns[stateParamPattern] || stateParamPattern,
        stateParamKey: stateParamKey,
        displayName: displayName
    };

    var detailParentRoute = listConfig ? list : parent;
    var editParentRoute = viewConfig ? view : detailParentRoute;

    return {
        detailParentRoute: detailParentRoute,
        editParentRoute: editParentRoute,
        routeDefinition: routeDefinition
    };
}

var RouteSet = function RouteSet(_ref2) {
    var name = _ref2.name;
    var baseUrl = _ref2.url;
    var _ref2$templateRequire = _ref2.templateRequireContext;
    var templateRequireContext = _ref2$templateRequire === undefined ? require.context('./templates/') : _ref2$templateRequire;
    var _ref2$resource = _ref2.resource;
    var resource = _ref2$resource === undefined ? name : _ref2$resource;
    var _ref2$resourceName = _ref2.resourceName;
    var resourceName = _ref2$resourceName === undefined ? _pluralize2.default.plural(resource) : _ref2$resourceName;
    var _ref2$displayName = _ref2.displayName;
    var displayName = _ref2$displayName === undefined ? _pluralize2.default.singular(resourceName.split('/').pop()) : _ref2$displayName;
    var _ref2$stateParamKey = _ref2.stateParamKey;
    var stateParamKey = _ref2$stateParamKey === undefined ? _pluralize2.default.singular(resourceName.split('/').pop()) + 'Id' : _ref2$stateParamKey;
    var _ref2$stateParamPatte = _ref2.stateParamPattern;
    var stateParamPattern = _ref2$stateParamPatte === undefined ? 'int' : _ref2$stateParamPatte;
    var _ref2$parent = _ref2.parent;
    var parent = _ref2$parent === undefined ? null : _ref2$parent;
    var _ref2$injectAt = _ref2.injectAt;
    var injectAt = _ref2$injectAt === undefined ? '' + (parent || '') : _ref2$injectAt;
    var _ref2$list = _ref2.list;
    var list = _ref2$list === undefined ? {} : _ref2$list;
    var _ref2$create = _ref2.create;
    var create = _ref2$create === undefined ? {} : _ref2$create;
    var _ref2$view = _ref2.view;
    var view = _ref2$view === undefined ? {} : _ref2$view;
    var _ref2$edit = _ref2.edit;
    var edit = _ref2$edit === undefined ? {} : _ref2$edit;

    _classCallCheck(this, RouteSet);

    (0, _assert2.default)(name, 'You did not provide a name for this route.');
    (0, _assert2.default)(baseUrl, 'You did not set a url for this route');

    var _defineRoute = defineRoute({ name: name, parent: parent, displayName: displayName, stateParamKey: stateParamKey, stateParamPattern: stateParamPattern, list: list, create: create, view: view, edit: edit });

    var detailParentRoute = _defineRoute.detailParentRoute;
    var editParentRoute = _defineRoute.editParentRoute;
    var routeDefinition = _defineRoute.routeDefinition;


    if (list !== null) this.list = listRouteFactory({
        name: name,
        routeDefinition: routeDefinition,
        baseUrl: baseUrl,
        parent: parent,
        templateRequireContext: templateRequireContext,
        resourceName: resourceName,
        viewName: list.viewName || 'list',
        injectAt: injectAt
    }, list);

    if (create !== null) this.create = createRouteFactory({
        name: name,
        routeDefinition: routeDefinition,
        baseUrl: baseUrl,
        parent: detailParentRoute,
        templateRequireContext: templateRequireContext,
        resourceName: resourceName,
        viewName: create.viewName || 'create',
        injectAt: injectAt
    }, create);

    if (view !== null) this.view = viewRouteFactory({
        name: name,
        routeDefinition: routeDefinition,
        baseUrl: baseUrl,
        parent: detailParentRoute,
        templateRequireContext: templateRequireContext,
        resourceName: resourceName,
        viewName: view.viewName || 'detail',
        injectAt: injectAt
    }, view);

    if (edit !== null) this.edit = editRouteFactory({
        name: name,
        routeDefinition: routeDefinition,
        baseUrl: baseUrl,
        parent: editParentRoute,
        templateRequireContext: templateRequireContext,
        resourceName: resourceName,
        viewName: edit.viewName || 'edit',
        injectAt: injectAt
    }, edit);
};

function listRouteFactory(_ref3, _ref4) {
    var name = _ref3.name;
    var routeDefinition = _ref3.routeDefinition;
    var baseUrl = _ref3.baseUrl;
    var parent = _ref3.parent;
    var templateRequireContext = _ref3.templateRequireContext;
    var resourceName = _ref3.resourceName;
    var viewName = _ref3.viewName;
    var injectAt = _ref3.injectAt;
    var _ref4$template = _ref4.template;
    var template = _ref4$template === undefined ? templateRequireContext('./list.html') : _ref4$template;
    var _ref4$url = _ref4.url;
    var url = _ref4$url === undefined ? baseUrl : _ref4$url;
    var _ref4$controller = _ref4.controller;
    var controller = _ref4$controller === undefined ? _controllers.GenericListController : _ref4$controller;
    var _ref4$controllerAs = _ref4.controllerAs;
    var controllerAs = _ref4$controllerAs === undefined ? 'listCtrl' : _ref4$controllerAs;
    var _ref4$resolve = _ref4.resolve;
    var resolve = _ref4$resolve === undefined ? {} : _ref4$resolve;
    var _ref4$data = _ref4.data;
    var data = _ref4$data === undefined ? {} : _ref4$data;
    var _ref4$views = _ref4.views;
    var views = _ref4$views === undefined ? {} : _ref4$views;

    if (!resolve.records) {
        resolve.records = function resolveRecords($stateParams, Restangular, $state, $location) {
            // TODO need a way to add as parameter parent this.routes PKs?
            var route = this.getResource(Restangular, $stateParams).all(resourceName);
            if (this.params.page) {
                return route.customGET(null, { page: $stateParams.page, page_size: $stateParams.page_size });
            } else {
                return route.getList();
            }
        };
        resolve.records.$inject = ['$stateParams', 'Restangular', '$state', '$location'];
    }

    function getResource(Restangular, $stateParams) {
        if (parent && resourceName[0] != '/' && this.parent.getResource) return this.parent.getResource(Restangular, $stateParams);

        return Restangular;
    }

    views[viewName + '@' + injectAt] = {
        controller: controller,
        controllerAs: controllerAs,
        template: template,
        resolve: resolve
    };

    data.routeDefinition = routeDefinition;

    return {
        parent: parent,
        name: routeDefinition.list,
        url: url,
        views: views,
        data: data,
        resolve: resolve,
        getResource: getResource,
        defaultDisplayName: (0, _utils.capFirst)(_pluralize2.default.plural(name))
    };
}

function createRouteFactory(_ref5, _ref6) {
    var name = _ref5.name;
    var routeDefinition = _ref5.routeDefinition;
    var baseUrl = _ref5.baseUrl;
    var parent = _ref5.parent;
    var templateRequireContext = _ref5.templateRequireContext;
    var resourceName = _ref5.resourceName;
    var viewName = _ref5.viewName;
    var injectAt = _ref5.injectAt;
    var _ref6$createTemplate = _ref6.createTemplate;
    var createTemplate = _ref6$createTemplate === undefined ? tryLoadTemplate(templateRequireContext, './create.html') : _ref6$createTemplate;
    var _ref6$template = _ref6.template;
    var template = _ref6$template === undefined ? createTemplate || templateRequireContext('./edit.html') : _ref6$template;
    var _ref6$url = _ref6.url;
    var url = _ref6$url === undefined ? routeDefinition.list ? '/create' : baseUrl + '/create' : _ref6$url;
    var _ref6$controller = _ref6.controller;
    var controller = _ref6$controller === undefined ? _controllers.GenericDetailController : _ref6$controller;
    var _ref6$controllerAs = _ref6.controllerAs;
    var controllerAs = _ref6$controllerAs === undefined ? createTemplate ? 'createCtrl' : 'editCtrl' : _ref6$controllerAs;
    var _ref6$resolve = _ref6.resolve;
    var resolve = _ref6$resolve === undefined ? {} : _ref6$resolve;
    var _ref6$data = _ref6.data;
    var data = _ref6$data === undefined ? {} : _ref6$data;
    var _ref6$views = _ref6.views;
    var views = _ref6$views === undefined ? {} : _ref6$views;

    // Inject the resource into the controller to be able to catch it in its closure.
    if (!resolve.record) {
        resolve.record = function resolveRecords($stateParams, Restangular) {
            // TODO need a way to add as parameter parent routes PKs?
            return this.getResource(Restangular, $stateParams).all(resourceName);
        };
        resolve.record.$inject = ['$stateParams', 'Restangular'];
    }

    function getResource(Restangular, $stateParams) {
        if (parent && resourceName[0] != '/' && this.parent.getResource) return this.parent.getResource(Restangular, $stateParams);

        return Restangular;
    }

    views[viewName + '@' + injectAt] = {
        controller: controller,
        controllerAs: controllerAs,
        template: template
    };

    data.routeDefinition = routeDefinition;

    return {
        parent: parent,
        name: routeDefinition.create,
        url: url,
        views: views,
        data: data,
        resolve: resolve,
        getResource: getResource,
        defaultDisplayName: 'Create ' + (0, _utils.capFirst)(_pluralize2.default.singular(name))
    };
}

function viewRouteFactory(_ref7, _ref8) {
    var name = _ref7.name;
    var routeDefinition = _ref7.routeDefinition;
    var baseUrl = _ref7.baseUrl;
    var parent = _ref7.parent;
    var templateRequireContext = _ref7.templateRequireContext;
    var resourceName = _ref7.resourceName;
    var viewName = _ref7.viewName;
    var injectAt = _ref7.injectAt;
    var _ref8$template = _ref8.template;
    var template = _ref8$template === undefined ? templateRequireContext('./view.html') : _ref8$template;
    var _ref8$url = _ref8.url;
    var url = _ref8$url === undefined ? (routeDefinition.list ? '' : baseUrl) + ('/{' + routeDefinition.stateParamKey + ':' + routeDefinition.stateParamPattern + '}') : _ref8$url;
    var _ref8$controller = _ref8.controller;
    var controller = _ref8$controller === undefined ? _controllers.GenericDetailController : _ref8$controller;
    var _ref8$controllerAs = _ref8.controllerAs;
    var controllerAs = _ref8$controllerAs === undefined ? 'viewCtrl' : _ref8$controllerAs;
    var _ref8$resolve = _ref8.resolve;
    var resolve = _ref8$resolve === undefined ? {} : _ref8$resolve;
    var _ref8$data = _ref8.data;
    var data = _ref8$data === undefined ? {} : _ref8$data;
    var _ref8$views = _ref8.views;
    var views = _ref8$views === undefined ? {} : _ref8$views;

    if (!resolve.record) {
        resolve.record = function resolveRecords($stateParams, Restangular, $state) {
            // TODO need a way to add as parameter parent routes PKs?
            return this.getResource(Restangular, $stateParams).get();
        };
        resolve.record.$inject = ['$stateParams', 'Restangular', '$state'];
    }

    function getResource(Restangular, $stateParams) {
        var parentResource = Restangular;
        if (parent && resourceName[0] != '/' && this.parent.getResource) parentResource = this.parent.getResource(Restangular, $stateParams);

        return parentResource.one(resourceName, $stateParams[routeDefinition.stateParamKey]);
    }

    views[viewName + '@' + injectAt] = {
        controller: controller,
        controllerAs: controllerAs,
        template: template
    };

    data.routeDefinition = routeDefinition;

    return {
        parent: parent,
        name: routeDefinition.view,
        url: url,
        views: views,
        data: data,
        resolve: resolve,
        getResource: getResource,
        defaultDisplayName: (0, _utils.capFirst)(_pluralize2.default.singular(name))
    };
}

function editRouteFactory(_ref9, _ref10) {
    var name = _ref9.name;
    var routeDefinition = _ref9.routeDefinition;
    var baseUrl = _ref9.baseUrl;
    var parent = _ref9.parent;
    var templateRequireContext = _ref9.templateRequireContext;
    var resourceName = _ref9.resourceName;
    var viewName = _ref9.viewName;
    var injectAt = _ref9.injectAt;
    var _ref10$template = _ref10.template;
    var template = _ref10$template === undefined ? templateRequireContext('./edit.html') : _ref10$template;
    var _ref10$url = _ref10.url;
    var url = _ref10$url === undefined ? routeDefinition.view ? '/edit' : '/' + baseUrl + '/{' + routeDefinition.stateParamKey + ':' + routeDefinition.stateParamPattern + '}/edit' : _ref10$url;
    var _ref10$controller = _ref10.controller;
    var controller = _ref10$controller === undefined ? _controllers.GenericDetailController : _ref10$controller;
    var _ref10$controllerAs = _ref10.controllerAs;
    var controllerAs = _ref10$controllerAs === undefined ? 'editCtrl' : _ref10$controllerAs;
    var _ref10$resolve = _ref10.resolve;
    var resolve = _ref10$resolve === undefined ? {} : _ref10$resolve;
    var _ref10$data = _ref10.data;
    var data = _ref10$data === undefined ? {} : _ref10$data;
    var _ref10$views = _ref10.views;
    var views = _ref10$views === undefined ? {} : _ref10$views;


    // Resolve from parent route
    // TODO if there is no view route we need to resolve directly.
    resolve.record = ['record', function (record) {
        return record;
    }];

    views[viewName + '@' + injectAt] = {
        controller: controller,
        controllerAs: controllerAs,
        template: template
    };

    data.routeDefinition = routeDefinition;

    return {
        parent: parent,
        name: routeDefinition.edit,
        url: url,
        views: views,
        data: data,
        resolve: resolve,
        defaultDisplayName: 'Edit ' + (0, _utils.capFirst)(_pluralize2.default.singular(name))
    };
}

exports.default = RouteSet;